# Oomph Paragraphs

Reusable paragraph bundles for Drupal 8.

## Getting Started

This module is only available currently as a private repository in Oomph, Inc's
organization on GitHub. In order to include this module in your project you will
need to add a new repository to your Composer configuration and install composer
from a device that is setup to access our private repostitories over SSH.

### Requirements

This module requires your project to be running Drupal 8 and PHP 7.1+. A list of dependencies can be found in [composer.json][] and [oomph_paragraphs.info.yml][].

### Installation

In your terminal require `drupal/oomph_paragraphs` with Composer:

```bash
$ composer require drupal/oomph_paragraphs
```

If you're using a project that's setup with `composer/installers` listening for
the `drupal-module` package type, then the module will be installed in the
`modules/contrib` directory of your Drupal project.

## Paragraph Bundles

Currently five (5) pre-configured paragraph bundles are included with this
module. The paragraph types that ship with this module are configured based on a
component design.

### Row

The Row is the only component that supports a background image, and if provided,
the row will maintain the aspect ratio of the image provided in modern browsers
(think of this design option like a traditional "hero" at the top of the page,
where text can overlay a large image).

### Column Group

Layout inside of a row is done with CSS flexbox. Components that sit inside a
row will be treated as columns when a layout is applied. In some circumstances,
an author may want to group two or more components together to be treated as a
single column. The column group allows this, and supports any number or type of
component inside, to be treated as a single column entity in the Layout chosen.

### WYSIWYG

The wysiwyg component is similar to the "body" field found by default on Drupal
content types. This allows content editors to style text with in a CKEditor
field with similar controls to a word processing application.

### Image

The image component accepts image uploads. The default configuration that ships
with this module currently uses the `image` field, however, any image fields can
be removed and replaced with a `responsive_image` field with the same machine
name for the field.

### Video

The video paragraph component uses the `video_embed_field` module to embed
videos into content types. This allows for content editors to upload a video
from a 3rd party provider.

## Configuration

This module ships with "optional" configuration. That means it will install
when the module is enabled only if that set of configuration does not already
exist in Drupal.

### Updating configuration for this module

If the need arises to update configuration for this module using the Configuration Management UI in Drupal or using Drush, the configuration files exported will include a
`uuid` value associated with the site's configuration. The Configuration
Installer component in Drupal cannot import configuration from a module with a
`uuid` value.

A script is defined with this module's `composer.json` to automatically strip
the `uuid` values in the configuration files located in `config/optional`.

```bash
$ composer remove-config-uuid
```

## TODOs

- [x] Revisit structual styles created by @jhogue to reduce the amount of scss
      files to be compiled and group styles in individual component files.
- [x] Register compiled styles in `oomph_paragraphs.libraries.yml` and load
      component styles in the compontent twig templates using
      `{{ attach_library('oomph-paragraphs/[component_name]') }}`
- [x] Investigate what bootstrap components are dependencies of the styles for
      this module.
- [x] Change style selectors from `.par__` to `.paragraph__`.
- [ ] Clean up Twig template code in template files.
- [x] Investigate if this module can be uploaded to drupal.org so we can easily
      require this module in our projects.
- [ ] Create process of contributing to this project and create process for
      release management.
- [ ] Investigate if `response_image` fields should be shipped with this module
      and if we should create a default set of responsive image configuration.

[composer.json]: composer.json
[oomph_paragraphs.info.yml]: oomph_paragraphs.info.yml
